@@ -121,22 +121,25 @@ int ipa_ccm_idtag_parse_off(struct tlv_parsed *dec, unsigned char *buf, int len,
 
 	memset(dec, 0, sizeof(*dec));
 
+	LOGP(DLMI, LOGL_DEBUG, "Rx IPA CCM ID_GET: ");
 	while (len >= 2) {
 		len -= 2;
 		t_len = *cur++;
 		t_tag = *cur++;
 
 		if (t_len < len_offset) {
+			LOGPC(DLMI, LOGL_DEBUG, "\n");
 			LOGP(DLMI, LOGL_ERROR, "minimal offset not included: %d < %d\n", t_len, len_offset);
 			return -EINVAL;
 		}
 
 		if (t_len > len + 1) {
+			LOGPC(DLMI, LOGL_DEBUG, "\n");
 			LOGP(DLMI, LOGL_ERROR, "The tag does not fit: %d > %d\n", t_len, len + 1);
 			return -EINVAL;
 		}
 
-		DEBUGPC(DLMI, "%s='%s' ", ipa_ccm_idtag_name(t_tag), cur);
+		LOGPC(DLMI, LOGL_DEBUG, "%s='%s' ", ipa_ccm_idtag_name(t_tag), cur);
 
 		dec->lv[t_tag].len = t_len - len_offset;
 		dec->lv[t_tag].val = cur;
@@ -144,6 +147,7 @@ int ipa_ccm_idtag_parse_off(struct tlv_parsed *dec, unsigned char *buf, int len,
 		cur += t_len - len_offset;
 		len -= t_len - len_offset;
 	}
+	LOGPC(DLMI, LOGL_DEBUG, "\n");
 	return 0;
 }
 
@@ -164,17 +168,19 @@ int ipa_ccm_id_get_parse(struct tlv_parsed *dec, const uint8_t *buf, unsigned in
 
 	memset(dec, 0, sizeof(*dec));
 
+	LOGP(DLMI, LOGL_DEBUG, "Rx IPA CCM ID_GET: ");
 	while (len >= 2) {
 		len -= 2;
 		t_len = *cur++;
 		t_tag = *cur++;
 
 		if (t_len > len + 1) {
+			LOGPC(DLMI, LOGL_DEBUG, "\n");
 			LOGP(DLMI, LOGL_ERROR, "The tag does not fit: %d > %d\n", t_len, len + 1);
 			return -EINVAL;
 		}
 
-		DEBUGPC(DLMI, "%s='%s' ", ipa_ccm_idtag_name(t_tag), cur);
+		LOGPC(DLMI, LOGL_DEBUG, "%s='%s' ", ipa_ccm_idtag_name(t_tag), cur);
 
 		dec->lv[t_tag].len = t_len-1;
 		dec->lv[t_tag].val = cur;
@@ -182,6 +188,7 @@ int ipa_ccm_id_get_parse(struct tlv_parsed *dec, const uint8_t *buf, unsigned in
 		cur += t_len-1;
 		len -= t_len-1;
 	}
+	LOGPC(DLMI, LOGL_DEBUG, "\n");
 	return 0;
 }
 
@@ -202,6 +209,7 @@ int ipa_ccm_id_resp_parse(struct tlv_parsed *dec, const uint8_t *buf, unsigned i
 
 	memset(dec, 0, sizeof(*dec));
 
+	LOGP(DLMI, LOGL_DEBUG, "Rx IPA CCM ID_RESP: ");
 	while (len >= 3) {
 		len -= 3;
 		t_len = osmo_load16be(cur);
@@ -209,6 +217,7 @@ int ipa_ccm_id_resp_parse(struct tlv_parsed *dec, const uint8_t *buf, unsigned i
 		t_tag = *cur++;
 
 		if (t_len > len + 1) {
+			LOGPC(DLMI, LOGL_DEBUG, "\n");
 			LOGP(DLMI, LOGL_ERROR, "The tag does not fit: %d > %d\n", t_len, len + 1);
 			return -EINVAL;
 		}
@@ -221,6 +230,7 @@ int ipa_ccm_id_resp_parse(struct tlv_parsed *dec, const uint8_t *buf, unsigned i
 		cur += t_len-1;
 		len -= t_len-1;
 	}
+	LOGPC(DLMI, LOGL_DEBUG, "\n");
 	return 0;
 }
 
@@ -257,34 +267,43 @@ int ipa_parse_unitid(const char *str, struct ipaccess_unit *unit_data)
 	return 0;
 }
 
+/*! Fill ud struct from tp structure.
+ *  \param[in,out] ud ipaccess_unit to fill
+ *  \param[in] tp the decoded TLV structure from eg. ID_RESP message
+ *  \returns zero on success, negative on error
+ *
+ * This function expects parameter ud's fields to be initialized to zero if not yet set.
+ * Existing incoming string pointer fields are expected to be allocated using
+ * talloc and will be deallocated as such if replaced with the content of tp.
+ **/
 int ipa_ccm_tlv_to_unitdata(struct ipaccess_unit *ud,
 			     const struct tlv_parsed *tp)
 {
 	int rc = 0;
 
 	if (TLVP_PRES_LEN(tp, IPAC_IDTAG_SERNR, 1))
-		ud->serno = talloc_strdup(ud, (char *)
-					TLVP_VAL(tp, IPAC_IDTAG_SERNR));
+		osmo_talloc_replace_string(ud, &ud->serno,
+					   (char *)TLVP_VAL(tp, IPAC_IDTAG_SERNR));
 
 	if (TLVP_PRES_LEN(tp, IPAC_IDTAG_UNITNAME, 1))
-		ud->unit_name = talloc_strdup(ud, (char *)
-					TLVP_VAL(tp, IPAC_IDTAG_UNITNAME));
+		osmo_talloc_replace_string(ud, &ud->unit_name,
+					   (char *)TLVP_VAL(tp, IPAC_IDTAG_UNITNAME));
 
 	if (TLVP_PRES_LEN(tp, IPAC_IDTAG_LOCATION1, 1))
-		ud->location1 = talloc_strdup(ud, (char *)
-					TLVP_VAL(tp, IPAC_IDTAG_LOCATION1));
+		osmo_talloc_replace_string(ud, &ud->location1,
+					   (char *)TLVP_VAL(tp, IPAC_IDTAG_LOCATION1));
 
 	if (TLVP_PRES_LEN(tp, IPAC_IDTAG_LOCATION2, 1))
-		ud->location2 = talloc_strdup(ud, (char *)
-					TLVP_VAL(tp, IPAC_IDTAG_LOCATION2));
+		osmo_talloc_replace_string(ud, &ud->location2,
+					   (char *)TLVP_VAL(tp, IPAC_IDTAG_LOCATION2));
 
 	if (TLVP_PRES_LEN(tp, IPAC_IDTAG_EQUIPVERS, 1))
-		ud->equipvers = talloc_strdup(ud, (char *)
-					TLVP_VAL(tp, IPAC_IDTAG_EQUIPVERS));
+		osmo_talloc_replace_string(ud, &ud->equipvers,
+					   (char *)TLVP_VAL(tp, IPAC_IDTAG_EQUIPVERS));
 
 	if (TLVP_PRES_LEN(tp, IPAC_IDTAG_SWVERSION, 1))
-		ud->swversion = talloc_strdup(ud, (char *)
-					TLVP_VAL(tp, IPAC_IDTAG_SWVERSION));
+		osmo_talloc_replace_string(ud, &ud->swversion,
+					   (char *)TLVP_VAL(tp, IPAC_IDTAG_SWVERSION));
 
 	if (TLVP_PRES_LEN(tp, IPAC_IDTAG_MACADDR, 17)) {
 		rc = osmo_macaddr_parse(ud->mac_addr, (char *)
@@ -378,7 +397,7 @@ struct msgb *ipa_ccm_make_id_resp(const struct ipaccess_unit *dev,
 		tag = msgb_put(msg, 3 + strlen(str) + 1);
 		tag[0] = 0x00;
 		tag[1] = 1 + strlen(str) + 1;
-		tag[2] = ies_req[1];
+		tag[2] = ies_req[i];
 		memcpy(tag + 3, str, strlen(str) + 1);
 	}
 	ipa_prepend_header(msg, IPAC_PROTO_IPACCESS);
@@ -402,10 +421,14 @@ struct msgb *ipa_ccm_make_id_resp_from_req(const struct ipaccess_unit *dev,
 	/* build a array of the IEIs */
 	while (len >= 2) {
 		uint8_t t_len, t_tag;
-		len -= 2;
+		len -= 2; /* subtract the length of the two bytes read below */
 		t_len = *cur++;
 		t_tag = *cur++;
 
+		/* as the 'tag' is included in the length of t_len, this cannot happen */
+		if (t_len == 0)
+			break;
+
 		if (t_len > len + 1) {
 			LOGP(DLINP, LOGL_ERROR, "IPA CCM tag 0x%02x does not fit\n", t_tag);
 			break;
@@ -413,13 +436,14 @@ struct msgb *ipa_ccm_make_id_resp_from_req(const struct ipaccess_unit *dev,
 
 		ies[num_ies++] = t_tag;
 
-		cur += t_len;
+		/* we need to subtract one from t_len to account for the tag */
+		cur += t_len - 1;
 		/* prevent any unsigned integer underflow due to somebody sending us
 		 * messages with wrong length values */
 		if (len <= t_len)
-			len -= t_len;
-		else
 			len = 0;
+		else
+			len -= t_len - 1;
 	}
 	return ipa_ccm_make_id_resp(dev, ies, num_ies);
 }
@@ -463,7 +487,7 @@ int ipa_ccm_rcvmsg_base(struct msgb *msg, struct osmo_fd *bfd)
 	case IPAC_MSGT_PING:
 		ret = ipa_ccm_send_pong(bfd->fd);
 		if (ret < 0) {
-			LOGP(DLINP, LOGL_ERROR, "Cannot send PING "
+			LOGP(DLINP, LOGL_ERROR, "Cannot send PONG "
 			     "message. Reason: %s\n", strerror(errno));
 			break;
 		}
