@@ -25,6 +25,8 @@
 
 #define _GNU_SOURCE
 
+#include "config.h"
+
 #include <string.h>
 #include <stdlib.h>
 #include <errno.h>
@@ -87,7 +89,7 @@ static struct cmd_node sched_node = {
 };
 
 /* returns number of configured CPUs in the system, or negative otherwise */
-static int get_num_cpus() {
+static int get_num_cpus(void) {
 	static unsigned int num_cpus = 0;
 	long ln;
 
@@ -637,8 +639,10 @@ int osmo_cpu_sched_vty_apply_localthread(void)
 		return 0;
 	}
 
+#ifdef HAVE_PTHREAD_GETNAME_NP
 	if (pthread_getname_np(pthread_self(), name, sizeof(name)) == 0)
 		has_name = true;
+#endif
 
 	/* Get latest matching mask for the thread */
 	pthread_mutex_lock(&sched_vty_opts->cpu_affinity_li_mutex);
